import contextlib
import sys

import adbc_driver_flightsql.dbapi
import adbc_driver_manager


class KamuConnection(contextlib.ExitStack):
    def __init__(self, url):
        super().__init__()

        self.url = url

        self._adbc = adbc_driver_flightsql.dbapi.connect(
            self.url,
            db_kwargs={
                adbc_driver_manager.DatabaseOptions.USERNAME.value: "kamu",
                adbc_driver_manager.DatabaseOptions.PASSWORD.value: "kamu",
            },
            autocommit=True,
        )

    def as_adbc(self):
        """
        Returns the underlying ADBC connection.

        Use this method when working with libraries that expect ADBC connection.

        Examples
        --------
        >>> import pandas
        >>> import kamu
        >>>
        >>> with kamu.connect() as con:
        >>>     pandas.read_sql("select 1", con.as_adbc())
        """
        return self._adbc

    def __enter__(self):
        super().__enter__()

        try:
            self.enter_context(self._adbc)
        except:
            if not self.__exit__(*sys.exc_info()):
                raise

        return self

    def __exit__(self, exc_type, exc_value, traceback):
        super().__exit__(exc_type, exc_value, traceback)



from . import _connection
from ._version import VERSION


def connect(url):
    """
    Open connection to a Kamu node.

    Examples
    --------
    >>> import kamu
    >>>
    >>> # Connect to secure node
    >>> with kamu.connect("grpc+tls://node.demo.kamu.dev:50050") as con:
    >>>     pass
    >>>
    >>> # Connect to local insecure node
    >>> with kamu.connect("grpc://localhost:50050") as con:
    >>>     pass
    """
    return _connection.KamuConnection(url=url)



import re

import pandas

import kamu


def test_version():
    assert re.fullmatch(
        r"\d\.\d\.\d", kamu.VERSION
    ), "Version doesn't match the pattern"


def test_sql_query_minimal(container_mt):
    with kamu.connect(container_mt.url) as con:
        actual = pandas.read_sql_query("select 1 as value", con.as_adbc())
        expected = pandas.DataFrame({"value": [1]})
        pandas.testing.assert_frame_equal(expected, actual)


def test_sql_query_dataset(container_mt):
    with kamu.connect(container_mt.url) as con:
        actual = pandas.read_sql_query(
            r"""
            select
                offset,
                op,
                reported_date,
                id,
                gender,
                age_group,
                location
            from 'kamu/covid19.british-columbia.case-details.hm'
            order by offset
            limit 1
            """,
            con.as_adbc(),
        )

        expected = pandas.DataFrame(
            {
                "offset": [0],
                "op": [0],
                "reported_date": ["2020-01-29T00:00:00Z"],
                "id": [1],
                "gender": ["M"],
                "age_group": ["40s"],
                "location": ["Out of Canada"],
            }
        ).astype(
            dtype={
                "offset": "int64",
                "op": "int32",
                "reported_date": "datetime64[ms, UTC]",
                "id": "int64",
            }
        )

        pandas.testing.assert_frame_equal(expected, actual)
